@@ -8,19 +8,7 @@ from typing import Callable
 from magicapi.Models.Call import make_call_from_request_and_response
 from magicapi.Models.task_executor import save_tasks
 
-import os
-import sentry_sdk
-
-dsn = os.getenv("SENTRY_DSN")
-print("DSN in middleware", dsn)
-sentry_sdk.init(
-    dsn,
-    # Set traces_sample_rate to 1.0 to capture 100%
-    # of transactions for performance monitoring.
-    # We recommend adjusting this value in production.
-    traces_sample_rate=1.0,
-)
-from sentry_sdk import capture_exception
+from magicapi.Services.Sentry import HAS_SENTRY_DSN, capture_exception
 
 
 class MagicCallLogger(APIRoute):
@@ -34,7 +22,8 @@ class MagicCallLogger(APIRoute):
             try:
                 response: Response = await original_route_handler(request)
             except Exception as e:
-                capture_exception(e)
+                if HAS_SENTRY_DSN:
+                    capture_exception(e)
                 if "aws.context" in request.scope:
                     context = request.scope["aws.context"]
                     context.serverless_sdk.capture_exception(e)
