@@ -1,3 +1,4 @@
+from typing import Optional
 import requests
 from functools import wraps
 
@@ -23,8 +24,12 @@ LOCATION, PROJECT_ID, DOORMAN_ID = (
     g.settings.doorman_public_project_id,
 )
 
-ID_TOKEN_ENDPOINT: str = f"https://{LOCATION}-{PROJECT_ID}.cloudfunctions.net/getIdToken"
-DOORMAN_BACKEND_ENDPOINT: str = "https://sending-messages-for-doorman.herokuapp.com/phoneLogic"
+ID_TOKEN_ENDPOINT: str = (
+    f"https://{LOCATION}-{PROJECT_ID}.cloudfunctions.net/getIdToken"
+)
+DOORMAN_BACKEND_ENDPOINT: str = (
+    "https://sending-messages-for-doorman.herokuapp.com/phoneLogic"
+)
 
 doorman_prefix = "/doorman"
 bare_token_path = f"{doorman_prefix}/token"
@@ -32,6 +37,7 @@ token_url = (
     bare_token_path if g.settings.local else f"/{g.settings.stage}{bare_token_path}"
 )
 oath2_scheme = OAuth2PasswordBearer(tokenUrl=token_url)
+oath2_scheme_or_none = OAuth2PasswordBearer(tokenUrl=token_url, auto_error=False)
 
 doorman_router = APIRouter(route_class=MagicCallLogger)
 
@@ -62,9 +68,27 @@ def login_with_phone(phone_number: PhoneNumber):
     return resp
 
 
+def sign_in_with_magic_link(form_data):
+    body = {"email": form_data.username, "magicLink": form_data.password}
+
+    magic_link_id_token_endpoint = ID_TOKEN_ENDPOINT.replace(
+        "getIdToken", "getIdTokenFromMagicLink"
+    )
+
+    id_resp = requests.post(magic_link_id_token_endpoint, json=body).json()
+    id_token = id_resp.get("idToken")
+    if not id_token:
+        print(id_resp)
+        raise DoormanAuthException(message=str(id_resp))
+
+    return {"access_token": id_token, "token_type": "bearer"}
+
+
 @doorman_router.post("/token")
 @need_doorman_vars
 def login(form_data: OAuth2PasswordRequestForm = Depends()):
+    if "@" in form_data.username:
+        return sign_in_with_magic_link(form_data)
     body = {
         "action": "verifyCode",
         "phoneNumber": form_data.username,
@@ -86,18 +110,31 @@ def login(form_data: OAuth2PasswordRequestForm = Depends()):
     return {"access_token": id_token, "token_type": "bearer"}
 
 
-@need_firestore
-def get_current_user(token: str = Depends(oath2_scheme)):
+def get_current_user_from_token(token: str):
     try:
         decoded = auth.verify_id_token(token)
         current_user = CurrentUser(**decoded)
         return current_user
     except firebase_admin._token_gen.ExpiredIdTokenError as e:
+        print("DOORMAN ERROR", e)
         raise DoormanAuthException(message=e)
 
 
 @need_firestore
-def get_current_email_user(token: str = Depends(oath2_scheme)):
+def get_current_user(token: str = Depends(oath2_scheme)):
+    return get_current_user_from_token(token=token)
+
+
+@need_firestore
+def get_current_user_or_none(token: Optional[str] = Depends(oath2_scheme_or_none)):
+    if not token:
+        return None
+    current_user = get_current_user_from_token(token)
+    return current_user
+
+
+@need_firestore
+def get_current_user_raw(token: str = Depends(oath2_scheme)):
     try:
         decoded = auth.verify_id_token(token)
         return decoded
@@ -106,5 +143,6 @@ def get_current_email_user(token: str = Depends(oath2_scheme)):
 
 
 GET_USER = Depends(get_current_user)
+GET_USER_OR_NONE = Depends(get_current_user_or_none)
 
 g.app.include_router(doorman_router, prefix=doorman_prefix, tags=["Doorman"])
